<?php
// カレンダー＆予定表
/*
   ●管理者パスワードは、calendar.ini に記載されている。
   　変更したい際は、直接編集で変更できる。

   ToDo
    セーブファイルの分割読み込み NEW!
    国民の休日バグ修正 OK
    先月・来月を編集可能にするための構造改革 OK
*/

$SERVERCHARSET='utf-8';  // サーバー上のファイル名などに使われている文字コード
$SCRIPTCHARSET='utf-8';  // このPHPスクリプトや、HTML出力時に使われる文字コード

@mb_internal_encoding($SCRIPTCHARSET); //文字化け対策
@set_time_limit(10);                   // スクリプト実行タイムアウト(秒)
@chdir(dirname(__FILE__));             // カレントフォルダをスクリプトが存在する位置に移動

header("Content-Type:text/html;charset={$SCRIPTCHARSET}");
@ob_end_flush();                       // 出力バッファリングを使用しない。

$script=basename(__FILE__); // このスクリプトのファイル名
$jumpaddr= ($script=='index.php') ? './' : $script;

// ──────────────────
// 初期設定 開始 {
// ──────────────────

//バージョン表記
$version=<<<END
Ver. 2017.1129.01
END;

// 1つのセルに表示するスケジュール文字数
$MAXSCHEDULECHAR=64;

// ──────────────────
// 関連ファイルの定義

$holidayfile='./holiday.dat';   // 休日定義ファイル
$savefile='./schedule.dat';     // 予定セーブファイル
$configfile='./calendar.ini';   // 各種設定ファイル (現在は管理者パスワード記録用)

// ──────────────────

// 外部CSSファイルを読み込む場合はここに記入する。
$CSSFILE='./design.css';

// ページ全体に適用するCSSを記述
$CSSSTYLE='';

// サーバー上のzipコマンドのフルパス
$zip='/usr/bin/zip';

// 曜日名定義
$weekname=array('日','月','火','水','木','金','土'); // 日本語表記
//$weekname=array('Sun','Mon','Tue','Wed','Thu','Fri','Sat'); // 英語表記

// 和暦の元号定義
// 1つ目の値は該当元号となった年月の1日の日付。
// 2つ目の値はオフセット。 西暦 +オフセット で西暦を和暦に換算する際に使用する。 
// （オフセットは 和暦-西暦 で求める。）
$gengo_list=array();
$gengo_list['平成']=array('1989/1/1',-1988); // 1989/1/8
$gengo_list['昭和']=array('1970/1/1',-1925); // PHPは1970以前を扱えないので左記のとおりとする。1926/12/25


// ──────────────────
// ユーザーエージェントから OS とブラウザを判定

$ua=isset($_SERVER['HTTP_USER_AGENT']) ? $_SERVER['HTTP_USER_AGENT'] : '-';
$vua=htmlspecialchars($ua);
$t=chk_browser($ua);
$client_browser=$t[0];
$client_os=chk_os($ua);
//echo "OS:{$client_os} Browser:{$client_browser}<br>\nUA:{$vua}<br>\n";

// ──────────────────
// 設定ファイルの解析
// 管理者パスワードの取得
// 月間予定最大値文字数の取得
$adminpass='hoge';
$maxmonthplanchar=255;
if(is_readable($configfile)){
	$t = parse_ini_file($configfile , true);
	if(isset($t['general']['adminpass'])){ $adminpass=$t['general']['adminpass']; }else{ show_error('adminpass の設定が '.$configfile.' 内に見つかりません。<br>動作を停止します。'); }
	if(isset($t['general']['maxmonthplanchar'])){ $maxmonthplanchar=$t['general']['maxmonthplanchar']; }else{ show_error('maxmonthplanchar の設定が '.$configfile.' 内に見つかりません。<br>動作を停止します。'); }
}else{ // 設定ファイルが見つからない!
	htmlHeader('エラー：カレンダー');
	echo "<h3>システムエラー</h3>\n<p>設定ファイル {$configfile} が読み込めません。<br>動作を停止します。</p>\n";
	htmlFooter();
	@flush();
	exit;
}
unset($t);

// ------------
// 画面表示用HTMLの定義 START { ----------------

// 予定入力画面用フォーム
$closetext=vwin_toggle(); // バーチャルウィンドウを閉じるHTML記述を生成

$h1=($client_os=='ios'||$client_os=='android') ? '' : jsh_encode_wrap("上記の内容で予定を保存します。\n(Ctrl+S)");
if($client_os=='osx'){ $h1=jsh_encode_wrap("上記の内容で予定を保存します。\n(Ctrl+S または ⌘+S)"); }
$h2=($client_os=='ios'||$client_os=='android') ? '' : jsh_encode_wrap("入力を取り消します。\n(ESC または Ctrl+W)");
$editform=<<<END
<div class="editwin" style="">
<h3>[[date]] の予定</h3>
<form acton="{$jumpaddr}" method="post" name="vwinform" id="vwinform" class="dlg_form">
<textarea id="txt" name="txt" onMouseOver="document.vwinform.txt.focus()" class="dlg_textarea">[[schedule_text]]</textarea>
<input type="hidden" name="act" value="set">
<input type="hidden" name="y" value="[[year]]">
<input type="hidden" name="m" value="[[month]]">
<input type="hidden" name="ey" value="[[edit_year]]">
<input type="hidden" name="em" value="[[edit_month]]">
<input type="hidden" name="ed" value="[[edit_day]]">
<div class="dlg_lb"><input type="button" value="消去" onClick="this.form.txt.value='';"></div>
<div class="dlg_rb"><input type="submit" id="editwin_submit" value="　ＯＫ　"{$h1}>　　<input type="button" id="editwin_close" value="キャンセル" {$closetext}{$h2}></div>
</form>
</div>
END;
$editform=str_replace("\n",'',$editform); // 改行を削除して、余分な行が描画されないようにする

// 各種設定 → バックアップとリストア 画面用フォーム
$v_version=htmlspecialchars($version);
$settingform=<<<END
<div class="editwin">
<h3>バックアップとリストア</h3>
<form class="dlg_form" name="vwinform" id="vwinform">
<p>全てのスケジュールをバックアップ、リストア（復元）できます。</p>
<table border="0" cellspacing="0" cellpadding="4">
<tr><td>バックアップする</td><td><input type="button" value="設定" onClick="location.href='./?ph=dl&y=[[year]]&m=[[month]]';"></td></tr>
<tr><td>リストアする</td><td><input type="button" value="設定"  onClick="location.replace('./?ph=up&y=[[year]]&m=[[month]]')"></td></tr>
</table>
END;
if($adminpass!=''){ // パスワードが設定されている時は、ログアウトインタフェースを追加。
$settingform.=<<<END
<h3>ログアウト</h3>
<p>ログアウトすれば、次回アクセス時に管理用パスワードの入力が必要になります。</p>
<p><input type="button" value="ログアウト"  onClick="location.replace('./?ph=lo')"></p>
END;
}
$settingform.=<<<END
<h3>バージョン情報</h3>
<p>{$v_version}</p>
<div class="dlg_rb"><input type="button" id="editwin_close" value="　閉じる　" {$closetext}></div>
<input type="hidden" name="act" value="upset">
<input type="hidden" name="y" value="[[year]]">
<input type="hidden" name="m" value="[[month]]">
</form>
</div>
END;


// 年月指定ジャンプ画面用フォーム
$nowy=date('Y'); $nowm=date('n');
$h1=jsh_encode_wrap("年月フォームの内容を{$nowy}年{$nowm}月にリセットします。");
$gotoform=<<<END
<div class="editwin">
<h3>年月を指定して移動</h3>
<form class="dlg_form" name="vwinform" id="vwinform">
<p>指定月に移動できます。</p>
<table border="0" cellspacing="0" cellpadding="4">
<tr><td>年</td><td><input type="text" value="[[year]]" name="year" id="year" size="6" maxlength="4" min="1980" max="2037" onClick="this.select();" style="ime-mode:disabled;width:80px;"></td></tr>
<tr><td>月</td><td><input type="text" value="[[month]]" name="month" id="month" size="4" maxlength="2" min="1" max="12" onClick="this.select();" style="ime-mode:disabled;width:80px;"></td></tr>
</table>

<p><input type="button" name="jump" value="　移動　"  onClick="location.replace('./?y='+document.vwinform.year.value+'&m='+document.vwinform.month.value);">　　<input type="button" value="今月にリセット"  onClick="document.vwinform.year.value={$nowy};document.vwinform.month.value={$nowm};"{$h1}></p>

<div class="dlg_rb"><input type="button" id="editwin_close" value="　閉じる　" {$closetext}></div>
</form>
</div>
END;


unset($closetext);

// 画面表示用HTMLの定義 END } ----------------

// ──────────────────
// 初期設定 終了 }
// ──────────────────

// ──────────────────
// CGIパラメータの解析

// 表示する年
$nen = isset($_GET['y']) ? (int)$_GET['y'] : date('Y');   // 年 2015
if(isset($_POST['y'])){ $nen=(int)$_POST['y']; }
if($nen<1980){ $nen=1980; } if($nen>2037){ $nen=2037; }
// 1970年と2038年に存在する日付表現の限界に達すると、前後の月の処理が面倒なので、1980～2037年以外は扱わないことにする。

// 表示する月
$tuki = isset($_GET['m']) ? (int)$_GET['m'] : date('n');  // 月 1 － 12
if(isset($_POST['m'])){ $tuki = (int)$_POST['m']; }
if($tuki<1){$tuki=1;} if($tuki>12){$tuki=12;}

// スケジュール編集対象の年
$edit_year = isset($_GET['ey']) ? (int)$_GET['ey'] : date('Y');
if(isset($_POST['ey'])){ $edit_year = (int)$_POST['ey']; }
if($edit_year<1980){$edit_year=1980;} if($edit_year>2037){$edit_year=2037;}

// スケジュール編集対象の月
$edit_month = isset($_GET['em']) ? (int)$_GET['em'] : date('n');
if(isset($_POST['em'])){ $edit_month = (int)$_POST['em']; }
if($edit_month<1){$edit_month=1;} if($edit_month>12){$edit_month=12;}

// スケジュール編集対象の日
$edit_day = isset($_GET['ed']) ? (int)$_GET['ed'] : date('j');
if(isset($_POST['ed'])){ $edit_day = (int)$_POST['ed']; }
if($edit_day<1){$edit_day=1;} if($edit_day>99){$edit_day=99;}

$phase = isset($_GET['ph']) ? trim($_GET['ph']) : ''; // 画面フェーズ
if(isset($_POST['phase'])){ $phase=trim($_POST['phase']); }

$action=''; // 動作フラグ
if(isset($_POST['act'])){ $action=trim($_POST['act']);}

$schedule_text='';
if(isset($_POST['txt'])){ $schedule_text=trim($_POST['txt']);}



// ──────────────────
// パスワード認証用クッキー操作
$get_adminpass='';
$orgphase='';

if($adminpass!=''){
	if(isset($_POST['pwd'])){ $get_adminpass =trim($_POST['pwd']); } // POSTで入力パスワード指定があるなら採用。
	elseif(isset($_COOKIE['p'])){ $get_adminpass = text_decode($_COOKIE['p']); } // クッキーが有るなら、記録されたパスワードを取得。

	if($get_adminpass!=$adminpass){ // パスワード無し or 不一致 → パスワード認証画面へ。
		setcookie('p','',1); // 誤ったクッキーをクリア
		$orgphase=$phase; // 現在指定されている画面フェーズをバックアップ
		$phase='auth';    // 画面フェーズをパスワード認証画面に強制変更。
		if($get_adminpass!=''){ draw_dialog('管理者パスワードが正しくありません。'); }
	}else{ // パスワード一致＝Cookieの有効期限を更新
		@setcookie('p',text_encode($get_adminpass),time()+60*60*24*60);
	}
}
// adminpass が設定されていない＝空 のときは認証画面とクッキーに関して何もしない

// ──────────────────
// 必要な設定ファイルが存在しない場合作成する。

if(!file_exists($holidayfile)){ @touch($holidayfile); }
if(!file_exists($savefile)){ @touch($savefile); }



// ──────────────────
// フェーズ定義 開始 {
// ──────────────────
// カレンダービューモード(デフォルト)
if($phase==''){

	// ──────────────────
	// スケジュールデータがリストアされた直後なら
	// スケジュールデータを読み込む前に、アップロードされたファイルをschedule.datに上書きする。
	if($action=='upset'){
		$stamp=date('Y_md_His'); // 2015_0101_235959
		if(!file_exists('./backup')){ mkdir('./backup'); @chmod('./backup',0755); }
		if(is_uploaded_file($_FILES["upfile"]["tmp_name"])){
			copy('schedule.dat',"./backup/schedule{$stamp}.dat"); // 念のため現在のデータをバックアップ
			if(file_exists("./backup/schedule{$stamp}.dat")) exec("{$zip} -m ./backup/schedule{$stamp}.zip ./backup/schedule{$stamp}.dat > /dev/null &"); // そしてzip圧縮保存
			if(move_uploaded_file($_FILES["upfile"]["tmp_name"], './schedule.dat')){ // 一時ファイルを schedule.dat に上書き
				@chmod('./schedule.dat', 0644);
				draw_dialog('スケジュールデータがリストアされました。');
			}else{
				draw_dialog('スケジュールデータがリストアできませんでした。');
			}
		}else{
			draw_dialog('リストア用ファイルが選択されていません。');
		}
	}

	// ──────────────────
	// 休日データをファイルから読み取る

	$holiday_ary=array();         // x月y日タイプの休日情報       $holiday_ary['md0101']='元旦';
	$holiday_week_ary=array();    // x月y週月曜 タイプの休日情報  $holiday_week_ary['mw0102']='成人の日';

	$buf=file($holidayfile);
	if(count($buf)>0){
		foreach($buf as $val){
			$val=trim(preg_replace('/[\s]+/',' ',$val)); // テキストを整形する
			if( preg_match('/^[#;]/',$val) || $val==''){ continue; } // 先頭が # や ; で始まる行や空行は無視する
			list($m,$d,$text,)=explode(' ',$val);
			if(preg_match('/^w/i',$d)){
				$d=preg_replace('/[^0-9]+/','',$d);
				$id_mw=sprintf("mw%02d%02d",$m,$d); // 月週をつなげた固有ID mw0228
				$holiday_week_ary[$id_mw]=$text;
			}else{
				$id_md=sprintf("md%02d%02d",$m,$d); // 月日をつなげた固有ID md0228
				$holiday_ary[$id_md]=$text;
			}
		}
	}

	// ──────────────────
	// 春分の日と秋分の日を計算で求め、読み取った休日データ配列に、上書きする。
	// 計算方法の出典: http://www.wanichan.com/pc/excel/2010/5/page07.html http://oshiete.goo.ne.jp/qa/1454974.html

	$day_haru=(int)(20.8431+0.242194*($nen-1980)-(int)(($nen-1980)/4));
	$id_md=sprintf("md%02d%02d",3,$day_haru);
	$holiday_ary[$id_md]='春分の日';
	$day_aki=(int)(23.2488+0.242194*($nen-1980)-(int)(($nen-1980)/4));
	$id_md=sprintf("md%02d%02d",9,$day_aki);
	$holiday_ary[$id_md]='秋分の日';

	// ──────────────────
	// スケジュールデータをファイルから読み取る

	$schedule_ary=array();         // x月y日の予定テキスト       $schedule_ary['s20140101']='初詣<>●●に行く';

	$buf=file($savefile);
	if(count($buf)>0){
		foreach($buf as $val){
			$val=trim(preg_replace('/[\s]+/',' ',$val)); // テキスト整形：連続スペースを一つにまとめる
			if( preg_match('/^[#;]/',$val) || $val==''){ continue; } // 先頭が # や ; で始まる行や空行は無視する
			list($y,$m,$d,$text)=explode(' ',$val,4);
			$id=sprintf("s%04d%02d%02d",$y,$m,$d);    // 年月日をつなげた固有ID s20140101
			if(trim($text)!=''){ $schedule_ary[$id]=$text; }
		}
	}

	unset($buf); // ファイル読み取りバッファを開放

	// ──────────────────

	$flg_needwrite=0; // 新しいスケジュールを書き込む必要がある時：1

	$nowday = date('Ymd');// 今日の日付 20150228

	// もしスケジュール書き換えがsubmitされた直後なら、スケジュール配列を修正して、画面上のつじつまを合わせる。
	// 更にスケジュールファイルへの書き込みを予約する。 $flg_needwrite → 1  (ファイル書き込みは後で処理する。)
	if($action=='set'){
		$id=sprintf("s%04d%02d%02d",$edit_year,$edit_month,$edit_day);    // 年月日をつなげた固有ID s20140101
		if($id==''){
			draw_dialog('データ異常。編集対象の年月日が取得できません。スケジュール書き換えを中断しました。');
		}elseif($schedule_text==''){ // スケジュールがクリアされたら予定を消す
			if(isset($schedule_ary[$id])){ unset($schedule_ary[$id]); $flg_needwrite=1; }
		}else{
			$t=str_replace("\n",'<>',str_replace("\r\n","\n",$schedule_text));
			if( !isset($schedule_ary[$id]) || $schedule_ary[$id]!=$t ){ $schedule_ary[$id]=$t; $flg_needwrite=1; }
		}
	}

	// デバッガ
	/* echo "<pre>";
	print_r($_POST);
	print_r($schedule_ary);
	echo "</pre>\n"; */

	// 先月ボタンが押された場合の処理
	if(isset($_POST['sengetu']) && $_POST['sengetu']!=''){
		list($nen,$tuki)=calc_ym($nen,$tuki,-1);  // 先月の年月を計算。
	// 来月ボタンが押された場合の処理
	}elseif(isset($_POST['raigetu']) && $_POST['raigetu']!=''){
		list($nen,$tuki)=calc_ym($nen,$tuki,1);  // 来月の年月を計算。
	// 今月ボタンが押された場合の処理
	}elseif(isset($_POST['kongetu']) && $_POST['kongetu']!=''){
		$nen=date('Y');
		$tuki=date('n');
	}else{
		; // 上記以外の場合は、入力された年月ををそのまま使用。
	}


	$start_youbi = date('w', mktime(0, 0, 0,$tuki,1,$nen));  // 今月の初めの曜日。       0 (日曜)から 6 (土曜)
	$tuki_owari  = date('t', mktime(0, 0, 0,$tuki,1,$nen));  // その月の終わりの日数。   28 から 31


	// HTMLヘッダを描画
	htmlHeader($nen.'/'.$tuki.' カレンダー');

	echo "<div class=\"content\">\n";

	//draw_jswin(); // Javascriptポップアップや バーチャルウィンドウ用の DIV タグを描画

	// 編集画面でのショートカットキーを定義する
	// Javascriptエリア 開始 {
	echo "<SCRIPT language=\"JavaScript\">\n<!--\n";
echo <<<END
shortcut.add("Ctrl+S",function() { document.vwinform.submit(); });
shortcut.add("Meta+S",function() { document.vwinform.submit(); });  // Mac対応
shortcut.add("ESC",function() { document.vwinform.editwin_close.click(); });
shortcut.add("Ctrl+W",function() { document.vwinform.editwin_close.click(); });
shortcut.add("Ctrl+T",function() { document.navi.kongetu.click(); });
END;
	// shortcut.add("Ctrl+N",function() { document.navi.raigetu.click(); });
	// shortcut.add("Ctrl+P",function() { document.navi.sengetu.click(); });
	echo "\n//-->\n</SCRIPT>\n";
	// Javascriptエリア 終了 }

	// ナビゲーションフォーム描画(先月・来月)
	if($client_os=='ios'||$client_os=='android'){  // iPhone系ではポップアップがあるとタッチしづらくなるので、ポップアップテキストを使用しない。
		$sengetu_hint='';
		$raigetu_hint='';
		$kongetu_hint='';
		$reload_hint='';
		$dl_hint='';
		$ul_hint='';
		$setting_hint='';
		$wareki_hint='';
	}else{
		$sengetu_hint=jsh_encode_wrap('前の月へ移動');
		$raigetu_hint=jsh_encode_wrap('次の月へ移動');
		$kongetu_hint=jsh_encode_wrap('今月（'.date('Y').'年'.date('n').'月）に戻る (Ctrl+T)');
		$reload_hint=jsh_encode_wrap('最新の情報に更新する。');
		$setting_hint=jsh_encode_wrap('各種設定を行う。');
		$wareki_hint=jsh_encode_wrap(seireki2wareki($nen,$tuki)."年{$tuki}月<br>クリックで指定月にジャンプ。"); // 和暦表示
	}

$buf=str_replace(array('[[year]]','[[month]]'),array($nen,$tuki),$settingform);
$vwinhtml=vwin_toggle($buf,'各種設定');
$buf=str_replace(array('[[year]]','[[month]]'),array($nen,$tuki),$gotoform);
$vwinhtml_goto=vwin_toggle($buf,'年月を指定して移動');

//$v_version=jsh_encode_wrap($version);

// 画面上部インタフェース用HTMLを描画
echo <<<EOT
<div class="navigation">
<form action="{$jumpaddr}" method="post" style="margin:0px; padding:0px;" name="navi" id="navi">
<input type="submit" name="sengetu" value="<<"{$sengetu_hint}> <span {$vwinhtml_goto}{$wareki_hint}>{$nen}年 {$tuki}月</span> <input type="submit" name="raigetu" value=">>"{$raigetu_hint}>
　<input type="submit" name="kongetu" value="今"{$kongetu_hint}>　<input type="button" name="reload" value="再" onClick="location.replace('./?y={$nen}&m={$tuki}')"{$reload_hint}>
　<input type="button" value="設"{$vwinhtml}{$setting_hint}>
<input type="hidden" name="y" value="{$nen}"><input type="hidden" name="m" value="{$tuki}">
</form>
</div>
EOT;

//echo $client_os."<br>";
// 　<span{$v_version}>v</span>

	// TABLEの見出し行を描画
	echo "<div class=\"maintable\">\n<table width=\"100%\">\n<tr class=\"week_cell\"><th class=\"nitiyoubi\">日</th><th>月</th><th>火</th><th>水</th><th>木</th><th>金</th><th class=\"doyoubi\">土</th></tr>\n<tr>";
	// ここで一旦画面をフラッシュ
	@flush();

//TABLE本体を演算描画

	$cnt_cell = 0;   // 使用したテーブルセルの個数カウンタ 1～

	// テーブルの各セルに記載する日付を格納する配列 $ary_id=array('20140228','20140301','20140302'...);
	// 今月の予定に相当する99日目は無視されるので問題ない。
	$ary_id = make_idarray($nen,$tuki);

	// デバッガ
	//echo "<pre>";
	//print_r($ary_id);
	//echo "</pre>\n";

	// 休日判定ループ START { -----------------
	// 最終的な描画を行う前に、休日かどうかをあらかじめ判定する。
	$cnt_mon = 0;    // 月曜カウンタ 1～4 or 5  当月において第何月曜日であるかの数値。 「x月y週目の月曜が休日」の判定に使う。
	$flg_furikae=0;  // 振替休日フラグ。 翌日以降が振替休日になる条件を満たした場合に1。
	foreach($ary_id as $id){
		$y=substr($id,0,4);
		$m=substr($id,4,2);
		$d=substr($id,6,2);
		$w=date('w',mktime(0,0,0,$m,$d,$y));         // 曜日 0 (日曜) から 6 (土曜)
		$id_md=sprintf("md%02d%02d",$m,$d);          // 月日をつなげた固有ID md0228 休日判定に使う。
		// 月曜日特有の特殊処理
		// 「x月y週目の月曜が休日」パターンの休日に対応する
		if($w==1 && $m==$tuki){
			$cnt_mon++; // 月曜カウンタを1つ進める
			$id_mw=sprintf("mw%02d%02d",$m,$cnt_mon);    // 月週をつなげた固有ID mw0203 を更新する。
			// 条件にマッチしたら、ファイルから読み取った休日データに直接追記する。
			//echo "$m 月 $d 日 (月) id:{$id_mw}<br>\n";
			if(isset($holiday_week_ary[$id_mw])){
				$holiday_ary[$id_md]=isset($holiday_ary[$id_md]) ? $holiday_ary[$id_md].' '.$holiday_week_ary[$id_mw] : $holiday_week_ary[$id_mw];
			}
		}
		// 「振替休日」の特別処理
		// 定義：日曜日が休日の場合、次の日以降で最も近い平日が振替休日となる
		if($flg_furikae==0){ // 振替休日フラグが立っていない場合
			// もし今日が日曜日かつ休日なら、振替休日フラグ=1 とする。(翌日以降が振替休日になりうる。)
			if($w==0 && isset($holiday_ary[$id_md])){  $flg_furikae=1;  }
		}else{ // 振替休日フラグが立っている場合
			// もし今日が休日でないなら、今日を振替休日とし、振替休日フラグ=0に戻す。
			if( ! isset($holiday_ary[$id_md]) ){ $holiday_ary[$id_md]='振替休日'; $flg_furikae=0;  }
		}
	}
	unset($cnt_mon,$flg_furikae);

	
	// ひととおり祝日と振替休日が計算された後で、もう一度チェックし、国民の休日を計算する
	// 「国民の休日」定義：「国民の祝日」で前後を挟まれた日が、「国民の祝日」でない場合、「国民の休日」となる。（飛び石連休を回避する意図がある。）
	$flg_reach=0;
	foreach($ary_id as $id){
		$y=substr($id,0,4);
		$m=substr($id,4,2);
		$d=substr($id,6,2);
		$w=date('w',mktime(0,0,0,$m,$d,$y));         // 曜日 0 (日曜) から 6 (土曜)
		$id_md=sprintf("md%02d%02d",$m,$d);          // 月日をつなげた固有ID md0228 休日判定に使う。
		$id_md_tommorow=sprintf("md%04d",date('md',mktime(0,0,0,$m,$d+1,$y))); // 明日の月日をつなげた固有ID md0301
		if(isset($holiday_ary[$id_md])){ // 今日が祝日の場合
			$flg_reach=1;                // フラグを立てる
		}else{ // 今日が祝日でない場合
			if($flg_reach>0){ // 昨日が祝日だった
				if(isset($holiday_ary[$id_md_tommorow])){ // 明日も祝日なら、 国民の休日となる
					 $holiday_ary[$id_md]=isset($holiday_ary[$id_md]) ? $holiday_ary[$id_md].' '.'国民の休日' : '国民の休日';
				}
				$flg_reach=0;
			}
		}
	}
	unset($flg_reach);
	// 休日判定ループ END } -----------------


	// メインループ START { ----------------
	$cnt=1;
	$max_cell=count($ary_id);
	foreach($ary_id as $val){

		$flg_kyujitu=0; // 休日の時1になるフラグ
		$title=''; // 日付の隣に書く備考

		$y=substr($val,0,4);
		$m=substr($val,4,2);
		$d=substr($val,6,2);
		if($d==99){continue;}
		$vd=number_format($d);
		$w=date('w',mktime(0,0,0,$m,$d,$y));         // 曜日 0 (日曜) から 6 (土曜)

		$id    = sprintf("%04d%02d%02d",$y,$m,$d);  // 年月日をつなげた固有ID 20140228
		$id_md = sprintf("md%02d%02d",$m,$d);       // 月日をつなげた固有ID md0228 休日判定に使う。

		// 休日にヒットするかチェック
		if($flg_kyujitu==0 && isset($holiday_ary[$id_md])){
			$flg_kyujitu=1; $title=' '.htmlspecialchars($holiday_ary[$id_md]);
		}


		// セルの背景色(CSSクラス)を決定する
		$td='';
		$td_cell='';
		if($nowday == $id){
			$td='honjitu'; //本日の色
			$td_cell='honjitu_cell';
		}elseif($m!=$tuki){ // 異なる月の色
			$td='blank';
			$td_cell='blank_cell';
		}elseif($flg_kyujitu>0){
			$td='kyujitu'; // 休日の色
			$td_cell='kyujitu_cell';
		}elseif($w == 6){
			$td= 'doyoubi'; //土曜日の色
			$td_cell='doyoubi_cell';
		}elseif($w == 0){
			$td= 'nitiyoubi'; //日曜日の色
			$td_cell='nitiyoubi_cell';
		}else{
			$td = 'heijitu'; //平日の色
			$td_cell='heijitu_cell';
		}
		// 予定入力用フォームを生成
		$schedule_id=sprintf("s%04d%02d%02d",$y,$m,$d);
		$sc_text=isset($schedule_ary[$schedule_id]) ? $schedule_ary[$schedule_id] : '';
		$vsc_text='<br>'.mb_strimwidth(nl2br(htmlspecialchars(str_replace("<>","\n",$sc_text))),0,$MAXSCHEDULECHAR,'...',$SCRIPTCHARSET);
		if($client_os=='ios'||$client_os=='android'){ // iPhone系ではポップアップがあるとタッチしづらくなるので、ポップアップテキストを使用しない。
			$poptxt='';
		}else{
			// ポップアップに表示するテキストpoptxtを準備  \記号はJavascript仕様に合わせ\\に変更
			$poptxt=$sc_text<>'' ? jsh_encode_wrap(htmlspecialchars(str_replace('<>',"\n",sprintf("%2d/%2d",$m,$d).'<>'.str_replace("\\","\\\\",$sc_text)))) : '';
		}
		// フォーム中のエンティティを置換
		$search =array('[[date]]','[[year]]','[[month]]','[[edit_year]]','[[edit_month]]','[[edit_day]]','[[schedule_text]]');
		$replace=array(sprintf("%d",$m).'月'.sprintf("%d",$d).'日 ('.$weekname[$w].')' , $nen , $tuki , $y, $m, $d, str_replace(array('<>',"\\"),array("\n","\\\\"),$sc_text));
		$buf=str_replace($search,$replace,$editform);
		$vwinhtml=vwin_toggle($buf,'予定入力'); // 入力用フォームを生成
		echo "<td valign=\"top\" class=\"{$td_cell}\"{$poptxt}{$vwinhtml}><span class=\"{$td}\">{$vd}{$title}</span>{$vsc_text}</td>"; // セルを描画


		// セル右端の折り返し処理 (ただし月末は別処理)
		if( ($w ==6) && ($cnt != $max_cell) ){
			echo "</tr>\n<tr>";     //折り返しを描画
		}

		$cnt++;
	}
	// メインループ END } ----------------


	echo "</tr>\n</table>\n";

	// 今月のその他予定
	$schedule_id=sprintf("s%04d%02d%02d",$nen,$tuki,99);
	$sc_text=isset($schedule_ary[$schedule_id]) ? $schedule_ary[$schedule_id] : '';
	$vsc_text=mb_strimwidth(nl2br(htmlspecialchars(str_replace("<>","\n",$sc_text))),0,$maxmonthplanchar,'...',$SCRIPTCHARSET);
	if($vsc_text==''){ $vsc_text='&nbsp;'; }
	if($client_os=='ios'||$client_os=='android'){ // iPhone系ではポップアップがあるとタッチしづらくなるので、ポップアップテキストを使用しない。
		$poptxt='';
	}else{
		// ポップアップに表示するテキストpoptxtを準備  \記号はJavascript仕様に合わせ\\に変更
		$poptxt=$sc_text<>'' ? jsh_encode_wrap(htmlspecialchars(str_replace('<>',"\n",str_replace("\\","\\\\",$sc_text)))) : '';
	}
	// フォーム中のエンティティを置換
	$search =array('[[date]]','[[year]]','[[month]]','[[edit_year]]','[[edit_month]]','[[edit_day]]','[[schedule_text]]');
	$replace=array(sprintf("%d",$tuki).'月' , $nen , $tuki , $nen, $tuki, 99, str_replace(array('<>',"\\"),array("\n","\\\\"),$sc_text));
	$buf=str_replace($search,$replace,$editform);
	$vwinhtml=vwin_toggle($buf,'予定入力'); // 入力用フォームを生成
	echo "<div class=\"etcplan\"{$poptxt}{$vwinhtml}>";
	echo "<div class=\"etctitle\">{$tuki}月のその他予定</div>\n";
	echo "{$vsc_text}</div>\n";

	echo "</div>\n"; // maintable おわり

	//echo "<div name=\"debugarea\" id=\"debugarea\"></div>"; // Javascriptからのデバッグ情報表示

	echo "</div>\n"; // content おわり


// デバッガ
//echo <<<END
//<form name="debug">
//<input type="text" value="" name="debug1" style="width:200px;"><br>
//<input type="text" value="" name="debug2" style="width:200px;"><br>
//<input type="text" value="" name="debug3" style="width:200px;"><br>
//<input type="text" value="" name="debug4" style="width:200px;">
//</form>
//END;

	echo "<div style=\"height:300px;\"></div>"; // ウインドウを開くのが不自然にならないようにするための下部ダミースペース
	@flush();



	// スケジュールの書き込みがsubmitされた直後のばあい、ファイルにスケジュールを書き込む。
	// HTML表示が終わった後で、ゆっくり処理する。
	// [！] 配列の内容全てをファイルに上書きする。この構造は、スケジュールの総量が増えた時にメモリ枯渇を招くかもしれない。
	if( $action=='set' && $flg_needwrite>0 ){
		$buf='';
		if(count($schedule_ary)>0){
			ksort($schedule_ary);
			foreach($schedule_ary as $i => $val){
				$y=substr($i,1,4);
				$m=substr($i,5,2);
				$d=substr($i,7,2);
				$ftxt=str_replace("\n",'<>',str_replace("\r\n","\n",$val)); // 改行コードを統一し、エンティティ<>に置き換える
				$buf.="{$y} {$m} {$d} {$ftxt}\n";
			}
		}
		if (is_writable($savefile)) {
			if (!$handle = fopen($savefile, 'w')) {
				draw_dialog("Cannot open file ($savefile)");
				exit;
			}

			if (fwrite($handle, $buf) === FALSE) {
				draw_dialog("Cannot write to file ($filename)");
				exit;
			}
			draw_dialog('スケジュールを更新しました。');
			fclose($handle);
		} else {
			echo "The file {$savefile} is not writable";
		}
	}


	htmlFooter();
	@flush();

// ──────────────────
// スケジュールデータのバックアップフェーズ
// ──────────────────
}elseif($phase=='dl'){
	// 直接ダウンロードに変更 2017/11/20
	if(!is_readable('./schedule.dat')){
		htmlHeader('カレンダー データバックアップ','',""); // <meta http-equiv=\"Refresh\" content=\"3;URL={$archname}\">
		echo "<div class=\"content\">";
		echo "<h3>バックアップエラー</h3>\n";
		echo "<p>スケジュールデータを読み取れません。</p>\n";
		echo "<p style=\"margin-top:2em\"><!-- もし自動的にダウンロードが始まらない場合は --><a href=\"./{$archname}\">こちらをクリックし保存してください。</a></p>\n";
		echo "<p><input type=\"button\" value=\"カレンダーに戻る\" onClick=\"location.replace('./?y={$nen}&m={$tuki}')\"></p>\n";
		echo "</div>\n";
		htmlFooter();
		exit();
	}
	$gzdata = gzencode(file_get_contents('./schedule.dat'), 9);
    header('Content-Description: File Transfer');
    header('Content-Type: application/octet-stream');
    header('Content-Disposition: attachment; filename="schedule.dat.gz"');
    header('Expires: 0');
    header('Cache-Control: must-revalidate');
    header('Pragma: public');
    header('Content-Length: ' . strlen($gzdata));
    echo $gzdata;
    exit;

// ──────────────────
// スケジュールデータのリストアフェーズ
// ──────────────────
}elseif($phase=='up'){

	htmlHeader('カレンダー データリストア');
	echo "<div class=\"content\">";
	echo "<h3>全スケジュールのリストア（復元）</h3>\n";
	echo "<p>スケジュールのデータをリストア（復元）するには schedule.dat をアップロードしてください。<br>\n";
	echo "アップロードすると<b>全スケジュールが完全に上書きされます。</b></p>\n";
	echo <<<END
<form acton="{$script}" method="post" enctype="multipart/form-data">
<p><input type="file" name="upfile" size="30"></p>
<input type="hidden" name="act" value="upset">
<input type="hidden" name="phase" value="">
<input type="hidden" name="y" value="{$nen}">
<input type="hidden" name="m" value="{$tuki}">
<p><input type="submit" value="アップロード"></p>
</form>
END;
	echo "<p style=\"margin-top:2em;\"><input type=\"button\" value=\"カレンダーに戻る\" onClick=\"location.replace('./?y={$nen}&m={$tuki}')\"></p>\n";
	echo "</div>\n";

	htmlFooter();

// ──────────────────
// パスワードを聞く画面
// ──────────────────
}elseif($phase=='auth'){

	htmlHeader('カレンダー パスワード認証');
	echo "<div class=\"content\">\n";
	echo "<h3>管理者パスワード確認</h3>\n";
	echo "<p>管理者パスワードを入力してください。</p>\n";
	echo <<<END
<form acton="{$script}" method="post">
<input type="password" name="pwd" size="30"></p>
<input type="hidden" name="phase" value="{$orgphase}">
<input type="hidden" name="y" value="{$nen}">
<input type="hidden" name="m" value="{$tuki}">
<input type="hidden" name="act" value="{$action}">
<input type="hidden" name="txt" value="{$schedule_text}">
<p><input type="submit" value="　ＯＫ　"></p>
</form>
</div>
END;

	htmlFooter();


// ──────────────────
// パスワードを聞く画面
// ──────────────────
}elseif($phase=='lo'){

	setcookie('p','',1); // クッキーをクリア

	htmlHeader('カレンダー ログアウト');
	echo "<div class=\"content\">\n";
	echo "<h3>ログアウトしました。</h3>\n";
	echo "<p><input type=\"button\" value=\"　戻る　\" onClick=\"location.replace('./')\"></p>\n";
	htmlFooter();


// ──────────────────
// その他想定外画面フェーズ
// ──────────────────
}else{
	htmlHeader('エラー：カレンダー');
	echo "<h3>フェーズエラー</h3>\n<p><a href=\"./\">戻ってください。</a></p>\n";
	htmlFooter();
	@flush();
}
// ──────────────────
// フェーズ定義 終了 }
// ──────────────────

exit;

// ──────────────────
// 関数定義 開始 {
// ──────────────────
// *	*	*	*	*	*	*	*	*	*	*	*	*
// HTMLヘッダ生成
// HTML5対応
/*
引数
	$html_title     ページタイトル
	$css            BODYタグに適用したいCSSスタイル
	$addhead        ヘッダの最後に追加される追加記述
*/
function htmlHeader($html_title='無題',$css='',$addhead='') {
global $SCRIPTCHARSET,$CSSFILE,$CSSSTYLE;

$csstxt='';
if($css!=''){ $csstxt=" style=\"{$css}\""; }

$cssfiletxt='';
if($CSSFILE!=''){ $cssfiletxt="<link rel=\"stylesheet\" type=\"text/css\" href=\"{$CSSFILE}\">\n"; }

$csshead='';
if($CSSSTYLE!=''){ $csshead="<style type=\"text/css\">\n<!--\n{$CSSSTYLE}\n-->\n</style>\n"; }

echo <<<END
<!DOCTYPE html>
<html lang="ja">
<head>
<meta charset="{$SCRIPTCHARSET}">
<link rel="shortcut icon" href="favicon.ico" >
<meta name="robots" content="noindex,nofollow,noarchive">
<meta name="viewport" content="initial-scale=1.0">
{$cssfiletxt}<script language="JavaScript" src="./popup.js"></script>
<link rel="stylesheet" type="text/css" href="./window.css">
<script language="JavaScript" src="./window.js"></script>
<script language="JavaScript" src="./mm_swapimage.js"></script>
<script language="JavaScript" src="./shortcut.js"></script>
{$csshead}<title>{$html_title}</title>
{$addhead}</head>
<body{$csstxt}>
END;

}
// 

// *	*	*	*	*	*	*	*	*	*	*	*	*
// HTMLフッタ生成
function htmlFooter() {
echo <<<END
<footer></footer>
</body>
</html>
END;

}

// *	*	*	*	*	*	*	*	*	*	*	*	*
// Javascriptポップアップや バーチャルウィンドウ用の DIV タグを描画
//function draw_jswin(){
//echo <<<END
//<div id="tiptext"></div>
//<div id="vwin"></div>
//END;
//}

// *	*	*	*	*	*	*	*	*	*	*	*	*
// 謎のエンコードの文字列をスクリプト文字コードに変換する
function conv_char($text){
	global $SCRIPTCHARSET,$SERVERCHARSET;
	$ret=$text;
	//$code=mb_detect_encoding($ret, 'ASCII,eucjp-win,sjis-win,utf-8');
	//if($code==''){$code=$SCRIPTCHARSET;} // コード判別不能の場合はとりあえずEUC
	//if($code!=$SCRIPTCHARSET) $ret=mb_convert_encoding($text,$SCRIPTCHARSET,$code); // 表示用ファイル名をコンバート
	if($SERVERCHARSET!=$SCRIPTCHARSET) $ret=mb_convert_encoding($text,$SCRIPTCHARSET,$SERVERCHARSET);
	return $ret;
}

// -------------------------------------------------------
// 改行のある文字列を JSH内で使えるようにエンコードする
// 改行を<br>に置き換え HTMLエンティティをエンコードして出力
function jsh_encode($txt=''){
	$ret=htmlspecialchars(str_replace(array("\r","\n","'"),array('','','\\\''),nl2br($txt)));
	return $ret;
}

// -------------------------------------------------------
// onMouseOver onMouseOut イベントのための記述を行い
// JSヘルプを表示するためのHTMLを文字列として返す
// jsh_encode 関数を再帰呼び出しする
function jsh_encode_wrap($txt=''){
	$ret='';
	if($txt) $ret=" onMouseMove=\"jspop('".jsh_encode($txt)."')\" onMouseOut=\"jspop()\" onMouseUp=\"jspop('')\"";
	return $ret;
}


// -------------------------------------------------------
// バーチャルウィンドウを表示・消去させる関数
function vwin_toggle($txt='',$title='カレンダー'){
	//$ret='';
	$ret=" onClick=\"w_pop('".htmlspecialchars(str_replace(array("\r","\n","'"),array('','\\n','\\\''),$txt))."','{$title}',10,50);\"";
	return $ret;
}

// -------------------------------------------------------
// 画面右上ダイアログボックスを描画する関数
function draw_dialog($txt=''){
	$vtxt=htmlspecialchars($txt);
	echo "<div class=\"dialog\" id=\"dialog\" onClick=\"element_invisible('dialog');\">{$vtxt}</div>\n";
	return;
}

// 画面に任意のメッセージを表示後にプログラムを終了する関数。
// 汎用画面の表示などに使う。
function show_mes($mes='',$page_title='',$css='',$addhead=''){
	htmlHeader($page_title,$css,$addhead);
	echo $mes;
	htmlFooter();
	@flush();
	exit;
}

// 画面にエラーメッセージを表示後にプログラムを終了する関数。
// エラーメッセージ表示を前提にして、show_mes()関数よりもシンプルな使い方。
function show_error($mes=''){
	htmlHeader('エラー：カレンダー');
	echo "<div class=\"content\"><h3>エラー</h3>\n<p>".$mes."</p></div>\n";
	htmlFooter();
	@flush();
	exit;
}


// -------------------------------------------------------
// ユーザーエージェントからOSを判定
// 戻り値： win95 win98 winme win2000 winxp winvista win2003 win7 win8 win81 winmobile winetc ios android osx mac
//          識別不能時 FALSE
function chk_os($ua=''){

$ret=FALSE;

if(preg_match('/Windows 95/i',$ua) || preg_match('/Win95/i',$ua)){
	$ret='win95';
}elseif(preg_match('/Windows 98/i',$ua) || preg_match('/Win98/i',$ua)){
	// if(preg_match('/98SE/i',$ua)){ $ret='win98se'; }
	$ret='win98';
}elseif(preg_match('/Windows 9x/i',$ua) || preg_match('/Win[ ]*9x/i',$ua)){
	$ret='winme';
}elseif( preg_match('/Windows[ ]*2000/i',$ua)){
	$ret='win2000';
}elseif(preg_match('/Windows NT 5\.1/i',$ua) || preg_match('/Windows[ ]*XP/i',$ua)){ // XPは NT 5.1
	$ret='winxp';
}elseif(preg_match('/Windows NT 5\.2/i',$ua)){
	$ret='win2003';
}elseif(preg_match('/Windows NT 5/i',$ua) || preg_match('/WinNT 5/i',$ua)){ // その他の Windows NT 5? は 2000扱い
	$ret='win2000';
}elseif(preg_match('/Windows NT 6\.0/i',$ua)){ // Vistaは NT 6.0
	$ret='winvista';
}elseif(preg_match('/Windows NT 6\.1/i',$ua)){  // 6.3 = Win8.1  6.2 = Win8  6.1 = Win7
	$ret='win7';
}elseif(preg_match('/Windows NT 6\.2/i',$ua)){ // Windows8 2015年2月9日 追加
	$ret='win8';
}elseif(preg_match('/Windows NT 6\.3/i',$ua)){ // Windows8.1 2015年2月9日 追加
	$ret='win81';
}elseif(preg_match('/Windows NT 10\.0/i',$ua)){ // Windows10 2015年10月5日 追加
	$ret='win10';
}elseif(preg_match('/Windows Mobile/i',$ua)){  // モバイル系
	$ret='winmobile';
}elseif(preg_match('/Windows/i',$ua)){ // その他よくわからないWindows
	$ret='winetc';
}elseif(preg_match('/(ipad|iphone).+ OS [0-9]+_[0-9]+/i',$ua)){ // Apple iOS 2015年2月9日 追加
	$ret='ios';
}elseif(preg_match('/android/i',$ua)){ // Android 2016年11月7日 追加
	$ret='android';
}elseif(preg_match('/Mac OS X/i',$ua)){ // Apple OS-X 2015年2月9日 追加
	$ret='osx';
}elseif(preg_match('/Macintosh/i',$ua)){
	$ret='mac';
}

return $ret;
}


// -------------------------------------------------------
// ユーザーエージェントからブラウザを判断  NextVから流用
// Windows Mobile対応のため若干修正 2011年11月5日
/*
戻り値  配列( ブラウザタイプ , Ver )
ブラウザタイプ：
opera-wm-old opera-wm opera firefox netscape chrome safari ie-wm ie
unknown
*/
function chk_browser($ua=''){
if(preg_match('/Opera[ \/][^ ]+/i',$ua,$match)){
 if(preg_match('/Windows[ ]*CE/i',$ua)){
  if(preg_match('/Opera[ ]*8.[56]/i',$ua)){
   return array('opera-wm-old',$match[0]); // プロキシ設定ありの古い Opera Ver.8.5x - 8.6x
  }else{
   return array('opera-wm',$match[0]); // プロキシ設定が無くOSのプロキシと共有の新しい Opera
  }
 }else{
  return array('opera',$match[0]); // その他Opera PC版？
 }
}elseif(preg_match('/Firefox[ \/]*[^ ]+/i',$ua,$match)){
 return array('firefox',$match[0]);
}elseif(preg_match('/Netscape[1-9, ,\/]*[^ ]+/i',$ua,$match)){
 return array('netscape',$match[0]);
}elseif(preg_match('/Edge[ \/]*[^ ]+/i',$ua,$match)){ // Microsoft Edge
 // EdgeはChromeやSafariと言うキーワードをUserAgent内に含むので、ChromeやSafariより前の段階でトラップする。2015年10月22日
 return array('edge',$match[0]);
}elseif(preg_match('/Chrome[ \/]*[^ ]+/i',$ua,$match)){
 return array('chrome',$match[0]);
}elseif(preg_match('/Safari[ \/]*[^ ]+/i',$ua,$match)){
 return array('safari',$match[0]);
}elseif(preg_match('/MSIE[ \/]*[^ ;]+/i',$ua,$match)){
 if(preg_match('/Windows[ ]*CE/i',$ua)){
  return array('ie-wm',$match[0]); // Windows Mobile
 }else{
  return array('ie',$match[0]);
 }
}elseif(preg_match('/Windows[ ]*NT.+Trident.+rv:11.+like Gecko/i',$ua,$match)){ // Windows IE11
 //IE11はMSIEと言うキーワードをUserAgent内に含まないため、別対応でIEと見なす。2014年10月29日  Mozilla/5.0 (Windows NT 6.3; WOW64; Trident/7.0; rv:11.0) like Gecko
 return array('ie',$match[0]);
}else{
 return array('unknown','Unknown');
}
}

// 西暦の数値を和暦に換算する。
// 現状は 平成 のことしか考えていない。面倒なので。
// 引数：
// $year      西暦年数値    (例) 2014
// 戻り値：
// $ret       和暦文字列  (例) "平成24"
function seireki2wareki($seireki='',$tuki=''){
	global $gengo_list;
	$ret='？？';
	$gengo='平成';
	$offset=0;
	if($seireki==''){ $seireki=date('Y'); }
	if($tuki==''){ $tuki=date('n'); }
	$target=strtotime("{$seireki}/{$tuki}/1");
	foreach($gengo_list as $key => $val){
		if(strtotime($val[0])<=$target){ $gengo=$key; $offset=$val[1]; break; }
	}
	// 和暦に変換
	$ret=$seireki+$offset;
	if($ret<=0){$ret="？";}
	elseif($ret==1){ $ret="元"; } // 1年目は元年
	$ret=$gengo.$ret;
	return $ret;
}


// 指定の月数ずらした年月を返す。
// 引数
// $year      基準となる西暦年数値      (例) 2014
// $month     基準となる月数値          1～12
// $offset    ずらす月数数値
// 戻り値：
// $ret       array(計算後の年,計算後の月)
// 例：
// calc_ym(2015,12,1)  →  array(2016,1)
// calc_ym(2014,1,-1)  →  array(2013,12)

//unixtimeスタンプは1970年から

function calc_ym($year=0,$month=0,$offset=0){
	if($year==0){ $year=date('Y'); }
	if($month==0){ $month=date('n'); }
	if($year<=1970 && $month+$offset<1 ){ $year=1970; $month=1; $offset=0; }
	if($year>=2037 && $month+$offset>12){ $year=2037; $month=12; $offset=0; }
	$ut=mktime(0, 0, 0, $month+$offset, 1, $year);
	$ry=date('Y',$ut);
	$rm=date('n',$ut);
	if($ry<1970){ $ry=1970; $rm=1; }
	if($ry>2037){ $ry=2037; $rm=12; }
	$ret=array($ry,$rm);
	return $ret;
}


// かんたん暗号化＆復号化
function text_decode($char=''){ return base64_decode(rawurldecode($char)); }
function text_encode($char=''){ return rawurlencode(base64_encode($char)); }


// テーブルの各セルに記載する日付ID yyyymmdd を生成する関数
// 戻り値： 配列 array('20140228','20140301','20140302'...);
function make_idarray($nen='',$tuki=''){
	if($nen==''){ $nen=date('Y'); }  if($tuki==''){ $tuki=date('m'); }
	$buf=array();

	$start_youbi = date('w', mktime(0, 0, 0,$tuki,1,$nen));  // 該当月の初めの曜日。       0 (日曜)から 6 (土曜)
	$tuki_owari  = date('t', mktime(0, 0, 0,$tuki,1,$nen));  // その月の終わりの日数。   28 から 31
	
	$month_offset=$start_youbi;                              // 当月の1日が始まるまでに、先月の日付が何日分存在するか。
	$max_cell=(int)ceil( ($month_offset + $tuki_owari) / 7 ) * 7 + 7; // TABLE描画に必要なセルの総数を計算  7の倍数になるよう調整した後、1週ぶん追加。(+7)
	
	for($i=0; $i<$max_cell; $i++){
		$buf[]=date('Ymd', mktime(0, 0, 0,$tuki,$i+1-$month_offset,$nen));
	}

	return $buf;
}


// ──────────────────
// 関数定義 終了 }
// ──────────────────
?>
